@@ -26,7 +26,7 @@ static const template i386_optab[] = {
 { "mov", 2, 0xb0, _, ShortFormW, {Imm, Reg, 0} },
 { "mov", 2, 0xc6, _,  W|Modrm,  {Imm, Reg|Mem, 0} },
 #ifdef NeXT_MOD /* bug fix for "movw %ds,64(%edx)" */
-{ "mov", 2, 0x8c, _, D|Modrm,  {SReg3|SReg2, Reg16|Mem, 0} },
+{ "mov", 2, 0x8c, _, D|Modrm,  {SReg3|SReg2, Reg|Mem, 0} },
 #else
 { "mov", 2, 0x8c, _, D|Modrm,  {SReg3|SReg2, Reg16|Mem16, 0} },
 #endif
@@ -102,7 +102,9 @@ static const template i386_optab[] = {
 {"lahf", 0, 0x9f, _, NoModrm, {0, 0, 0} },
 {"sahf", 0, 0x9e, _, NoModrm, {0, 0, 0} },
 {"pushf", 0, 0x9c, _, NoModrm, {0, 0, 0} },
+{"pushfd", 0, 0x9c, _, NoModrm, {0, 0, 0} },
 {"popf", 0, 0x9d, _, NoModrm, {0, 0, 0} },
+{"popfd", 0, 0x9d, _, NoModrm, {0, 0, 0} },
 {"stc", 0, 0xf9, _, NoModrm, {0, 0, 0} },
 {"std", 0, 0xfd, _, NoModrm, {0, 0, 0} },
 {"sti", 0, 0xfb, _, NoModrm, {0, 0, 0} },
@@ -476,41 +478,41 @@ static const template i386_optab[] = {
 
 #if defined(i486) || defined (i586)
 #define BSWAP_OPCODE 0x0fc8
-{"bswap",	1, 0x0fc8, _,    NoModrm,  {Reg32,           0,	0}, "4" },
-{"xadd",	2, 0x0fc0, _,  W|  Modrm,    {Reg,     Reg|Mem, 0}, "4" },
-{"cmpxchg",	2, 0x0fb0, _,  W|  Modrm,    {Reg,     Reg|Mem, 0}, "4" },
-{"invd",	0, 0x0f08, _,    NoModrm,      {0,           0,	0}, "4" },
-{"invlpg",	1, 0x0f01, 7,      Modrm,    {Mem,           0,	0}, "4" },
-{"wbinvd",	0, 0x0f09, _,    NoModrm,      {0,           0,	0}, "4" },
+{"bswap",	1, 0x0fc8, _,    NoModrm,  {Reg32,           0,	0}, /*"4"*/ },
+{"xadd",	2, 0x0fc0, _,  W|  Modrm,    {Reg,     Reg|Mem, 0}, /*"4"*/ },
+{"cmpxchg",	2, 0x0fb0, _,  W|  Modrm,    {Reg,     Reg|Mem, 0}, /*"4"*/ },
+{"invd",	0, 0x0f08, _,    NoModrm,      {0,           0,	0}, /*"4"*/ },
+{"invlpg",	1, 0x0f01, 7,      Modrm,    {Mem,           0,	0}, /*"4"*/ },
+{"wbinvd",	0, 0x0f09, _,    NoModrm,      {0,           0,	0}, /*"4"*/ },
 #endif /* defined (i486) || defined (i586) */
 
 #ifdef i586
 /* cmpxchg8b - here Mem32 means 32 bit pointer to a 64bit entity */
-{"cmpxchg8b",	1, 0x0fc7, _,       Modrm,    {Mem,           0, 0}, "5" },
-{"cpuid",	0, 0x0fa2, _,     NoModrm,      {0,           0, 0}, "5" },
-{"rdtsc",	0, 0x0f31, _,     NoModrm,      {0,           0, 0}, "5" },
-{"rdmsr",	0, 0x0f32, _,     NoModrm,      {0,           0, 0}, "5" },
-{"wrmsr",	0, 0x0f30, _,     NoModrm,      {0,           0, 0}, "5" },
-{"rsm",		0, 0x0faa, _,     NoModrm,      {0,           0, 0}, "5" },
+{"cmpxchg8b",	1, 0x0fc7, 1,       Modrm,    {Mem,           0, 0}, /*"5"*/ },
+{"cpuid",	0, 0x0fa2, _,     NoModrm,      {0,           0, 0}, /*"5"*/ },
+{"rdtsc",	0, 0x0f31, _,     NoModrm,      {0,           0, 0}, /*"5"*/ },
+{"rdmsr",	0, 0x0f32, _,     NoModrm,      {0,           0, 0}, /*"5"*/ },
+{"wrmsr",	0, 0x0f30, _,     NoModrm,      {0,           0, 0}, /*"5"*/ },
+{"rsm",		0, 0x0faa, _,     NoModrm,      {0,           0, 0}, /*"5"*/ },
 #endif /* i586 */
 
 #ifdef i686
 /* Pentium Pro extensions */
-{"rdpmc", 0, 0x0f33, _, NoModrm, {0, 0, 0}, "6" },
+{"rdpmc", 0, 0x0f33, _, NoModrm, {0, 0, 0}, /*"6"*/ },
         
-{"ud2", 0, 0x0fff, _, NoModrm, {0, 0, 0}, "6" }, /* official undefined instr. */
+{"ud2", 0, 0x0f0b, _, NoModrm, {0, 0, 0}, /*"6"*/ }, /* official undefined instr. */
 #endif /* i686 */
 
 #define Mem512 (Mem32|Disp|BaseIndex)
 
-  {"fxrstor", 1, 0x0fae, 1, Modrm, {Mem512, 0, 0}, "6" },
-  {"fxsave", 1, 0x0fae, 0, Modrm, {Mem512, 0, 0}, "6" },
-  {"sysenter", 0, 0x0f34, _, NoModrm, {0, 0, 0}, "6" },
-  {"sysexit", 0, 0x0f35, _, NoModrm, {0, 0, 0}, "6" },
+  {"fxrstor", 1, 0x0fae, 1, Modrm, {Mem512, 0, 0}, /*"6"*/ },
+  {"fxsave", 1, 0x0fae, 0, Modrm, {Mem512, 0, 0}, /*"6"*/ },
+  {"sysenter", 0, 0x0f34, _, NoModrm, {0, 0, 0}, /*"6"*/ },
+  {"sysexit", 0, 0x0f35, _, NoModrm, {0, 0, 0}, /*"6"*/ },
 
 #define def_cmov(name,opc) \
-  {name,2,opc,_,Modrm|ReverseRegRegmem,{Reg16|Mem16|Disp|BaseIndex,Reg16,0}, "6" }, \
-  {name,2,opc,_,Modrm|ReverseRegRegmem,{Reg32|Mem32|Disp|BaseIndex,Reg32,0}, "6" },
+  {name,2,opc,_,Modrm|ReverseRegRegmem,{Reg16|Mem16|Disp|BaseIndex,Reg16,0}, /*"6"*/ }, \
+  {name,2,opc,_,Modrm|ReverseRegRegmem,{Reg32|Mem32|Disp|BaseIndex,Reg32,0}, /*"6"*/ },
 
   def_cmov("cmova", 0x0f47)
   def_cmov("cmovae", 0x0f43)
@@ -528,6 +530,7 @@ static const template i386_optab[] = {
   def_cmov("cmovnbe", 0x0f47)
   def_cmov("cmovnc", 0x0f43)
   def_cmov("cmovne", 0x0f45)
+  def_cmov("cmovnz", 0x0f45)
   def_cmov("cmovng", 0x0f4e)
   def_cmov("cmovnge", 0x0f4c)
   def_cmov("cmovnl", 0x0f4d)
@@ -539,18 +542,18 @@ static const template i386_optab[] = {
   def_cmov("cmovp", 0x0f4a)
   def_cmov("cmovnp", 0x0f4b)
 
-  {"fcomi", 2, 0xdbf0, _, ShortForm, {FloatReg, FloatAcc, 0}, "6" },
-  {"fcomip", 2, 0xdff0, _, ShortForm, {FloatReg, FloatAcc, 0}, "6" },
-  {"fucomi", 2, 0xdbe8, _, ShortForm, {FloatReg, FloatAcc, 0}, "6" },
-  {"fucomip", 2, 0xdfe8, _, ShortForm, {FloatReg, FloatAcc, 0}, "6" },
-  {"fcmovb", 2, 0xdac0, _, ShortForm, {FloatReg, FloatAcc, 0}, "6" },
-  {"fcmove", 2, 0xdac8, _, ShortForm, {FloatReg, FloatAcc, 0}, "6" },
-  {"fcmovbe", 2, 0xdad0, _, ShortForm, {FloatReg, FloatAcc, 0}, "6" },
-  {"fcmovu", 2, 0xdad8, _, ShortForm, {FloatReg, FloatAcc, 0}, "6" },
-  {"fcmovnb", 2, 0xdbc0, _, ShortForm, {FloatReg, FloatAcc, 0}, "6" },
-  {"fcmovne", 2, 0xdbc8, _, ShortForm, {FloatReg, FloatAcc, 0}, "6" },
-  {"fcmovnbe", 2, 0xdbd0, _, ShortForm, {FloatReg, FloatAcc, 0}, "6" },
-  {"fcmovnu", 2, 0xdbd8, _, ShortForm, {FloatReg, FloatAcc, 0}, "6" },
+  {"fcomi", 2, 0xdbf0, _, ShortForm, {FloatReg, FloatAcc, 0}, /*"6"*/ },
+  {"fcomip", 2, 0xdff0, _, ShortForm, {FloatReg, FloatAcc, 0}, /*"6"*/ },
+  {"fucomi", 2, 0xdbe8, _, ShortForm, {FloatReg, FloatAcc, 0}, /*"6"*/ },
+  {"fucomip", 2, 0xdfe8, _, ShortForm, {FloatReg, FloatAcc, 0}, /*"6"*/ },
+  {"fcmovb", 2, 0xdac0, _, ShortForm, {FloatReg, FloatAcc, 0}, /*"6"*/ },
+  {"fcmove", 2, 0xdac8, _, ShortForm, {FloatReg, FloatAcc, 0}, /*"6"*/ },
+  {"fcmovbe", 2, 0xdad0, _, ShortForm, {FloatReg, FloatAcc, 0}, /*"6"*/ },
+  {"fcmovu", 2, 0xdad8, _, ShortForm, {FloatReg, FloatAcc, 0}, /*"6"*/ },
+  {"fcmovnb", 2, 0xdbc0, _, ShortForm, {FloatReg, FloatAcc, 0}, /*"6"*/ },
+  {"fcmovne", 2, 0xdbc8, _, ShortForm, {FloatReg, FloatAcc, 0}, /*"6"*/ },
+  {"fcmovnbe", 2, 0xdbd0, _, ShortForm, {FloatReg, FloatAcc, 0}, /*"6"*/ },
+  {"fcmovnu", 2, 0xdbd8, _, ShortForm, {FloatReg, FloatAcc, 0}, /*"6"*/ },
 
 /* SSE extensions */
 #define RR   ReverseRegRegmem
@@ -564,8 +567,10 @@ static const template i386_optab[] = {
 #define m8   (Mem32|Disp|BaseIndex)
 
   {"addpd", 2, 0x660f58, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
+  {"addsubpd", 2, 0x660fd0, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"addps", 2, 0x0f58, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"addsd", 2, 0xf20f58, _, RR|Modrm, {xmm|m64, xmm, 0},"O"},
+  {"addsubps", 2, 0xf20fd0, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"addss", 2, 0xf30f58, _, RR|Modrm, {xmm|m32, xmm, 0},"O"},
   {"andnpd", 2, 0x660f55, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"andnps", 2, 0x0f55, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
@@ -605,6 +610,16 @@ static const template i386_optab[] = {
   {"divsd", 2, 0xf20f5e, _, RR|Modrm, {xmm|m64, xmm, 0},"O"},
   {"divss", 2, 0xf30f5e, _, RR|Modrm, {xmm|m32, xmm, 0},"O"},
   {"emms", 0, 0x0f77, _, NoModrm, {0, 0, 0} },
+  {"fisttp", 1, 0xdf, 1, Modrm, {Mem, 0, 0},"O"},
+  {"fisttps", 1, 0xdf, 1, Modrm, {Mem, 0, 0},"O"},
+  {"fisttpl", 1, 0xdb, 1, Modrm, {Mem, 0, 0},"O"},
+  {"fisttpll", 1, 0xdd, 1, Modrm, {Mem, 0, 0},"O"},
+  {"haddpd", 2, 0x660f7c, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
+  {"haddps", 2, 0xf20f7c, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
+  {"hsubpd", 2, 0x660f7d, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
+  {"hsubps", 2, 0xf20f7d, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
+  {"lddqu", 2, 0xf20ff0, _, RR|Modrm, {m128, xmm, 0},"O"},
+  {"femms", 0, 0x0f0e, _, NoModrm, {0, 0, 0} },
   {"ldmxcsr", 1, 0x0fae, 2, Modrm, {m32, 0, 0},"O"},
   {"lfence", 0, 0x0faee8, _, NoModrm, {0, 0, 0},"O"},
   {"maskmovdqu", 2, 0x660ff7, _, RR|Modrm, {xmm, xmm, 0},"O"},
@@ -618,14 +633,18 @@ static const template i386_optab[] = {
   {"minps", 2, 0x0f5d, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"minsd", 2, 0xf20f5d, _, RR|Modrm, {xmm|m64, xmm, 0},"O"},
   {"minss", 2, 0xf30f5d, _, RR|Modrm, {xmm|m32, xmm, 0},"O"},
+  /* We only support the 0-operand version of monitor right now */
+  /* TBD: Support monitor %eax, %ecx, %edx */
+  {"monitor", 0, 0x0f01c8, _, NoModrm, {0, 0, 0},"O"},
   {"movapd", 2, 0x660f28, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"movapd", 2, 0x660f29, _, RR|Modrm, {xmm, xmm|m128, 0},"O"},
   {"movaps", 2, 0x0f28, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"movaps", 2, 0x0f29, _, RR|Modrm, {xmm, xmm|m128, 0},"O"},
   {"movd", 2, 0x0f6e, _, RR|Modrm, {r32|m32, mm, 0},"O"},
-  {"movd", 2, 0x0f7e, _, RR|Modrm, {mm, r32|m32, 0},"O"},
+  {"movd", 2, 0x0f7e, _, Modrm, {mm, r32|m32, 0},"O"},
   {"movd", 2, 0x660f6e, _, RR|Modrm, {r32|m32, xmm, 0},"O"},
-  {"movd", 2, 0x660f7e, _, RR|Modrm, {xmm, r32|m32, 0},"O"},
+  {"movd", 2, 0x660f7e, _, Modrm, {xmm, r32|m32, 0},"O"},
+  {"movddup", 2, 0xf20f12, _, RR|Modrm, {xmm|m64, xmm, 0},"O"},
   {"movdq2q", 2, 0xf20fd6, _, RR|Modrm, {xmm, mm, 0},"O"},
   {"movdqa", 2, 0x660f6f, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"movdqa", 2, 0x660f7f, _, RR|Modrm, {xmm, xmm|m128, 0},"O"},
@@ -650,11 +669,15 @@ static const template i386_optab[] = {
   {"movntq", 2, 0x0fe7, _, RR|Modrm, {mm, m64, 0},"O"},
   {"movq", 2, 0x0f6f, _, RR|Modrm, {mm|m64, mm, 0},"O"},
   {"movq", 2, 0x0f7f, _, RR|Modrm, {mm, mm|m64, 0},"O"},
-  {"movq", 2, 0x660fd6, _, RR|Modrm, {xmm|m64, xmm, 0},"O"},
-  {"movq", 2, 0xf30f7e, _, RR|Modrm, {xmm, xmm|m64, 0},"O"},
+  /* Both of these next opcodes can assemble a "movq xmm,xmm" but the first
+     must be used so the operand order is correctly encoded. */
+  {"movq", 2, 0xf30f7e, _, RR|Modrm, {xmm|m64, xmm, 0},"O"},
+  {"movq", 2, 0x660fd6, _, RR|Modrm, {xmm, xmm|m64, 0},"O"},
   {"movq2dq", 2, 0xf30fd6, _, RR|Modrm, {mm, xmm, 0},"O"},
   {"movsd", 2, 0xf20f10, _, RR|Modrm, {xmm|m64, xmm, 0},"O"},
   {"movsd", 2, 0xf20f11, _, RR|Modrm, {xmm, xmm|m64, 0},"O"},
+  {"movshdup", 2, 0xf30f16, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
+  {"movsldup", 2, 0xf30f12, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"movss", 2, 0xf30f10, _, RR|Modrm, {xmm|m32, xmm, 0},"O"},
   {"movss", 2, 0xf30f11, _, RR|Modrm, {xmm, xmm|m32, 0},"O"},
   {"movupd", 2, 0x660f10, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
@@ -665,6 +688,9 @@ static const template i386_optab[] = {
   {"mulps", 2, 0x0f59, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"mulsd", 2, 0xf20f59, _, RR|Modrm, {xmm|m64, xmm, 0},"O"},
   {"mulss", 2, 0xf30f59, _, RR|Modrm, {xmm|m32, xmm, 0},"O"},
+  /* We only support the 0-operand version of mwait right now */
+  /* TBD: Support mwait %eax, %ecx */
+  {"mwait", 0, 0x0f01c9, _, NoModrm, {0, 0, 0},"O"},
   {"orpd", 2, 0x660f56, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"orps", 2, 0x0f56, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"packssdw", 2, 0x0f6b, _, RR|Modrm, {mm|m64,mm,0},"O"},
@@ -693,9 +719,10 @@ static const template i386_optab[] = {
   {"pand", 2, 0x660fdb, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"pandn", 2, 0x0fdf, _, RR|Modrm, {mm|m64, mm, 0},"O"},
   {"pandn", 2, 0x660fdf, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
-  {"pause", 0, 0xf390, _, RR|NoModrm, {0, 0, 0},"O"},
+  {"pause", 0, 0xf390, _, RR|NoModrm, {0, 0, 0},},
   {"pavgb", 2, 0x0fe0, _, RR|Modrm, {mm|m64, mm, 0},"O"},
   {"pavgb", 2, 0x660fe0, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
+  {"pavgusb", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0xbf},
   {"pavgw", 2, 0x0fe3, _, RR|Modrm, {mm|m64, mm, 0},"O"},
   {"pavgw", 2, 0x660fe3, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"pcmpeqb", 2, 0x0f74, _, RR|Modrm, {mm|m64, mm, 0},"O"},
@@ -712,6 +739,27 @@ static const template i386_optab[] = {
   {"pcmpgtw", 2, 0x660f65, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"pextrw", 3, 0x0fc5, _, RR|Modrm, {Imm8, mm, r32},"O"},
   {"pextrw", 3, 0x660fc5, _, RR|Modrm, {Imm8, xmm, r32},"O"},
+  {"pf2id", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0x1d},
+  {"pf2iw", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0x1c},
+  {"pfacc", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0xae},
+  {"pfadd", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0x9e},
+  {"pfcmpeq", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0xb0},
+  {"pfcmpge", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0x90},
+  {"pfcmpgt", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0xa0},
+  {"pfmax", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0xa4},
+  {"pfmin", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0x94},
+  {"pfmul", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0xb4},
+  {"pfnacc", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0x8a},
+  {"pfpnacc", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0x8e},
+  {"pfrcp", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0x96},
+  {"pfrcpit1", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0xa6},
+  {"pfrcpit2", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0xb6},
+  {"pfrsqit1", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0xa7},
+  {"pfrsqrt", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0x97},
+  {"pfsub", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0x9a},
+  {"pfsubr", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0xaa},
+  {"pi2fd", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0x0d},
+  {"pi2fw", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0x0c},
   {"pinsrw", 3, 0x0fc4, _, RR|Modrm, {Imm8, r32|m16, mm},"O"},
   {"pinsrw", 3, 0x660fc4, _, RR|Modrm, {Imm8, r32|m16, xmm},"O"},
   {"pmaddwd", 2, 0x0ff5, _, RR|Modrm, {mm|m64, mm, 0},"O"},
@@ -726,6 +774,7 @@ static const template i386_optab[] = {
   {"pminub", 2, 0x660fda, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"pmovmskb", 2, 0x0fd7, _, RR|Modrm, {mm, r32, 0},"O"},
   {"pmovmskb", 2, 0x660fd7, _, RR|Modrm, {xmm, r32, 0},"O"},
+  {"pmulhrw", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0xb7},
   {"pmulhuw", 2, 0x0fe4, _, RR|Modrm, {mm|m64, mm, 0},"O"},
   {"pmulhuw", 2, 0x660fe4, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"pmulhw", 2, 0x0fe5, _, RR|Modrm, {mm|m64, mm, 0},"O"},
@@ -736,10 +785,12 @@ static const template i386_optab[] = {
   {"pmuludq", 2, 0x660ff4, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"por", 2, 0x0feb, _, RR|Modrm, {mm|m64, mm, 0},"O"},
   {"por", 2, 0x660feb, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
-  {"prefetcht0", 1, 0x0f18, 1, Modrm, {m8, 0, 0},"O"},
-  {"prefetcht1", 1, 0x0f18, 2, Modrm, {m8, 0, 0},"O"},
-  {"prefetcht2", 1, 0x0f18, 3, Modrm, {m8, 0, 0},"O"},
-  {"prefetchnta", 1, 0x0f18, 0, Modrm, {m8, 0, 0},"O"},
+  {"prefetch", 1, 0x0f0d, 0, Modrm, {m8, 0, 0} },
+  {"prefetchw", 1, 0x0f0d, 1, Modrm, {m8, 0, 0} },
+  {"prefetcht0", 1, 0x0f18, 1, Modrm, {m8, 0, 0} },
+  {"prefetcht1", 1, 0x0f18, 2, Modrm, {m8, 0, 0} },
+  {"prefetcht2", 1, 0x0f18, 3, Modrm, {m8, 0, 0} },
+  {"prefetchnta", 1, 0x0f18, 0, Modrm, {m8, 0, 0} },
   {"psadbw", 2, 0x0ff6, _, RR|Modrm, {mm|m64, mm, 0},"O"},
   {"psadbw", 2, 0x660ff6, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"pshufd", 3, 0x660f70, _, RR|Modrm, {Imm8, xmm|m128, xmm},"O"},
@@ -796,6 +847,7 @@ static const template i386_optab[] = {
   {"psubusw", 2, 0x660fd9, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"psubw", 2, 0x0ff9, _, RR|Modrm, {mm|m64, mm, 0},"O"},
   {"psubw", 2, 0x660ff9, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
+  {"pswapd", 2, 0x0f0f, _, RR|Modrm, {mm|m64, mm, 0},"O", 0xbb},
   {"punpckhbw", 2, 0x0f68, _, RR|Modrm, {mm|m64, mm, 0},"O"},
   {"punpckhbw", 2, 0x660f68, _, RR|Modrm, {xmm|m128, xmm, 0},"O"},
   {"punpckhdq", 2, 0x0f6a, _, RR|Modrm, {mm|m64, mm, 0},"O"},
@@ -973,8 +1025,8 @@ static const template i386_optab[] = {
 {"fsubr", 2, 0xdce8, _, ShortForm, {FloatAcc, FloatReg, 0} },
 #endif
 {"fsubr", 0, 0xdce9, _, NoModrm, {0, 0, 0} },
-{"fsubrp", 1, 0xdae8, _, ShortForm, {FloatReg, 0, 0} },
-{"fsubrp", 2, 0xdae8, _, ShortForm, {FloatReg, FloatAcc, 0} },
+{"fsubrp", 1, 0xdee8, _, ShortForm, {FloatReg, 0, 0} },
+{"fsubrp", 2, 0xdee8, _, ShortForm, {FloatReg, FloatAcc, 0} },
 #ifdef NON_BROKEN_OPCODES
 {"fsubrp", 2, 0xdee0, _, ShortForm, {FloatAcc, FloatReg, 0} },
 #else
