@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -159,7 +157,8 @@ main(
 int argc,
 char *argv[])
 {
-    unsigned long i, start, page_number;
+    int i, start;
+    unsigned long j, page_number;
     char *endp;
 
 	progname = argv[0];
@@ -182,8 +181,8 @@ char *argv[])
 
 	if(strcmp(argv[start], "-a") == 0){
 	    page_number = (ofile.file_size + vm_page_size - 1) / vm_page_size;
-	    for(i = 0; i < page_number; i++){
-		print_parts_for_page(i);
+	    for(j = 0; j < page_number; j++){
+		print_parts_for_page(j);
 	    }
 	    start++;
 	}
@@ -574,15 +573,15 @@ struct file_part *fp)
 		    if(symbols[i].n_type & N_EXT ||
 		       (fp->mh->filetype == MH_EXECUTE &&
 		        symbols[i].n_type & N_PEXT)){
-			if(symbols[i].n_un.n_strx > ext_high)
+			if((unsigned long)symbols[i].n_un.n_strx > ext_high)
 			    ext_high = symbols[i].n_un.n_strx;
-			if(symbols[i].n_un.n_strx < ext_low)
+			if((unsigned long)symbols[i].n_un.n_strx < ext_low)
 			    ext_low = symbols[i].n_un.n_strx;
 		    }
 		    else{
-			if(symbols[i].n_un.n_strx > local_high)
+			if((unsigned long)symbols[i].n_un.n_strx > local_high)
 			    local_high = symbols[i].n_un.n_strx;
-			if(symbols[i].n_un.n_strx < local_low)
+			if((unsigned long)symbols[i].n_un.n_strx < local_low)
 			    local_low = symbols[i].n_un.n_strx;
 		    }
 		}
@@ -1127,6 +1126,9 @@ unsigned long high_addr)
 {
     unsigned long i, count;
 
+	if(fp->st == NULL)
+	    return;
+
 	if(sorted_symbols == NULL)
 	    sorted_symbols = allocate(sizeof(struct nlist) * fp->st->nsyms);
 
